// Set up namespace and some state.
var mkdru = {
  // Settings to pass to pz2.js
  usesessions: Drupal.settings.mkdru.use_sessions === '1',
  //   showResponseType: 'json',
  // Variables
  submitted: false,
  pz2: null,
  totalRec: 0,
  sourceMax: Drupal.settings.mkdru.source_max,
  subjectMax: Drupal.settings.mkdru.subject_max,
  authorMax: Drupal.settings.mkdru.author_max,
  pazpar2Path: Drupal.settings.mkdru.pz2_path,
  // State
  defaultState: {
    page: 1,
    perpage: 20,
    sort: 'relevance',
    filter: null,
    query:null,
  },
  state: {},
};



// So we can use jQuery BBQ with Drupal 6 and its 1.2.6 jQuery
if (!$.isArray) $.isArray = function( obj ) {
  return toString.call(obj) === "[object Array]";
},



// pz2.js event handlers:
mkdru.pz2Init = function () {
  if (mkdru.state.query) mkdru.triggerSearch();
  //mkdru.pz2.stat();
  //mkdru.pz2.bytarget();
};

mkdru.pz2Show = function (data) {
  mkdru.totalRec = data.merged;

  var pagerHtml = Drupal.theme('mkdruPager', data, mkdru.state.page,
                                      Math.ceil(mkdru.totalRec / mkdru.state.perpage));
  $('.mkdru-pager').html(pagerHtml);
  $('.mkdru-next').bind('click', mkdru.nextPage);
  $('.mkdru-prev').bind('click', mkdru.prevPage);

  var countsHtml = 'Displaying: ' + (data.start + 1) + ' to '
    + (data.start + data.num) + ' of ' + data.merged + ' (found: ' + data.total + ')';
  $('.mkdru-counts').html(countsHtml);

  var html = "";
  for (var i = 0; i < data.hits.length; i++) {
    html += Drupal.theme('mkdruResult', data.hits[i], 
      i + 1 + mkdru.state.perpage * (mkdru.state.page - 1),
      "#" + $.param.fragment($.param.fragment(
        window.location.href, {recid: data.hits[i].recid})) + "\n"
    );
  }
  $('.mkdru-result-list').html(html);
  $('.mkdru-results').show();
};

mkdru.pz2Status = function (data) {
};

mkdru.pz2Term = function (data) {
  var html = "";
  for (var i = 0; i < data.xtargets.length && i < mkdru.sourceMax; i++ ) {
    html += Drupal.theme('mkdruTerm', data.xtargets[i].name, data.xtargets[i].freq, 
                          'mkdru-facet-source', data.xtargets[i].id);
  }
  $('.mkdru-facet-sources').html(html);

  html = "";
  for (var i = 0; i < data.subject.length && i < mkdru.subjectMax; i++ ) {
    html += Drupal.theme('mkdruTerm', data.subject[i].name, data.subject[i].freq, 
                          'mkdru-facet-subject', data.subject[i].id);
  }
  $('.mkdru-facet-subjects').html(html);

  html = "";
  for (var i = 0; i < data.author.length && i < mkdru.authorMax; i++ ) {
    html += Drupal.theme('mkdruTerm', data.author[i].name, data.author[i].freq, 
                          'mkdru-facet-author', data.author[i].id);
  }
  $('.mkdru-facet-authors').html(html);

  $('.mkdru-facet-source').bind('click', function (e) {
    mkdru.limitTarget(this.getAttribute('target_id'), this.firstChild.nodeValue);
    return false;
  });

  $('.mkdru-facet-subject').bind('click', function (e) {
    mkdru.limitQuery('su', this.firstChild.nodeValue);
    return false;
  });

  $('.mkdru-facet-author').bind('click', function (e) {
    mkdru.limitQuery('au', this.firstChild.nodeValue);
    return false;
  });
};

mkdru.pz2ByTarget = function (data) {
  
};

mkdru.pz2Record = function (data) {
  clearTimeout(mkdru.pz2.showTimer);
  $('.mkdru-results').hide();
  $('.mkdru-detail').html(Drupal.theme('mkdruDetail', data));
  $('.mkdru-detail-back').bind('click', function () {$.bbq.removeState('recid');});
  $('.mkdru-detail').show();
  clearTimeout(mkdru.pz2.recordTimer);
};



// State and URL handling 

// populate state from an object and fill in the blanks with defaults
mkdru.stateFromObject = function (obj) {
  for (var key in mkdru.defaultState) {
    if (typeof(obj[key]) != "undefined") {
      dump (key + " :: " + obj[key] + "\n");
      mkdru.state[key] = obj[key];
    }
    else {
      mkdru.state[key] = mkdru.defaultState[key];
    }
  }
};

// populate state from current window's hash string
mkdru.stateFromHash = function () {
  mkdru.stateFromObject($.deparam.fragment());
};

// set current window's hash string from state
mkdru.hashFromState = function () {
  // only include non-default settings in the URL
  var alteredState = {};
  for (var key in mkdru.defaultState) {
    if (mkdru.state[key] != mkdru.defaultState[key]) {
      alteredState[key] = mkdru.state[key];
    }
  }
  $.bbq.pushState(alteredState);
};

// update mkdru_form theme's ui to match state
mkdru.uiFromState = function () {
  for (var key in mkdru.state) {
    switch(key) {
    case 'query':
      $('.mkdru-search input:text').attr('value', mkdru.state[key]);
      break;
    case 'perpage':
      $('.mkdru-perpage').attr('value', mkdru.state[key]);
      break;
    case 'sort':
      $('.mkdru-sort').attr('value', mkdru.state[key]);
      break;
    }
  }
};

mkdru.hashChange = function () {
  dump("Submidded? " + mkdru.submitted + "\n");
  var hash = $.deparam.fragment();
  if (typeof(hash.recid) !== "undefined") {
    mkdru.pz2.record(hash.recid);
  }
  else {
    $('.mkdru-detail').hide();
    $('.mkdru-results').show();
  }
};



// UI functions:
mkdru.submitQuery = function () {
  mkdru.state.query = $('.mkdru-search input:text').attr('value');
  mkdru.resetPage();
  mkdru.pollDropDowns();
  mkdru.hashFromState();
  mkdru.search();
  mkdru.submitted = true;
  return false;
};

mkdru.triggerSearch = function () {
  mkdru.search();
  mkdru.submitted = true;
};

mkdru.search = function () {
  mkdru.pz2.search(mkdru.state.query, mkdru.state.perpage, mkdru.state.sort,
      mkdru.state.filter);
};

mkdru.pollDropDowns = function () {
  mkdru.state.perpage = $('.mkdru-perpage').attr('value');
  mkdru.state.sort = $('.mkdru-sort').attr('value');
  if (!mkdru.submitted) return false;
  mkdru.resetPage();
  mkdru.pz2.show(0, mkdru.state.perpage, mkdru.state.sort);
  return false;
};

mkdru.limitQuery = function (field, value) {
  $('.mkdru-search input:text').attr('value', function () {
    return this.value += ' and ' + field + '="' + value + '"';
  });
  mkdru.submitQuery();
};

mkdru.limitTarget = function (id, name) {
  var navi = document.getElementById('mkdru-navi');
  navi.innerHTML = 
        'Source: <a class="crossout" href="#" onclick="delimitTarget();return false;">'
        + name + '</a>';
  navi.innerHTML += '<hr/>';
  mkdru.state.filter = 'pz:id=' + id;
  mkdru.resetPage();
  mkdru.pollDropDowns();
  mkdru.search();
  return false;
};

mkdru.resetPage = function () {
  mkdru.state.page = 1;
  mkdru.totalRec = 0;
};

mkdru.showPage = function (pageNum) {
  mkdru.state.page = pageNum;
  mkdru.pz2.showPage(pageNum-1);
};

mkdru.nextPage = function () {
  if (mkdru.totalRec - mkdru.state.perpage * mkdru.state.page > 0) {
    mkdru.pz2.showNext();
    mkdru.state.page++;
    mkdru.hashFromState();
  }
};

mkdru.prevPage = function () {
  if (mkdru.pz2.showPrev() != false) {
    mkdru.state.page--;
    mkdru.hashFromState();
  }
};



// wait until the DOM is ready, bind events
// and instantiate pz2 library
$(document).ready(function () {
  $(window).bind( 'hashchange', mkdru.hashChange);
  $('.mkdru-search').bind('submit', mkdru.submitQuery);
  $('.mkdru-search input:text').attr('value', '');
  $('.mkdru-perpage').bind('change', function () { mkdru.pollDropDowns() });
  $('.mkdru-sort').bind('change', function () { mkdru.pollDropDowns() });

  dump("NEW PZ2!\n");
  mkdru.pz2 = new pz2( { "onshow": mkdru.pz2Show,
              "showtime": 500, //each timer (show, stat, term, bytarget) can be specified this way
              "pazpar2path": mkdru.pazpar2path,
              "oninit": mkdru.pz2Init,
              "onstat": mkdru.pz2Status,
              "onterm": mkdru.pz2Term,
              "termlist": "xtargets,subject,author",
              "onbytarget": mkdru.pz2ByTarget,
              "usesessions" : mkdru.usesessions,
              "showResponseType": mkdru.showResponseType,
              "onrecord": mkdru.pz2Record,
              "autoInit": false } );

  // initialise state to hash string or defaults
  mkdru.stateFromHash();
  // and update UI to match
  mkdru.uiFromState();
  dump("QUERY:" + mkdru.state.query + "\n");
  
  // ting thing
  if (typeof(Drupal.settings.mkdru.query) !== "undefined") {
    mkdru.state.query = Drupal.settings.mkdru.query
  }

  if (mkdru.usesessions) {
    mkdru.pz2.init();
  } else if (mkdru.state.query) {
    mkdru.triggerSearch(); 
  }
});
